<?php

namespace Drupal\advertising_products_amazon;

use Amazon\ProductAdvertisingAPI\v1\Configuration;
use Amazon\ProductAdvertisingAPI\v1\com\amazon\paapi5\v1\api\DefaultApi;
use Amazon\ProductAdvertisingAPI\v1\ApiException;
use Amazon\ProductAdvertisingAPI\v1\com\amazon\paapi5\v1\GetItemsRequest;
use Amazon\ProductAdvertisingAPI\v1\com\amazon\paapi5\v1\GetItemsResource;
use Amazon\ProductAdvertisingAPI\v1\com\amazon\paapi5\v1\PartnerType;
use Amazon\ProductAdvertisingAPI\v1\com\amazon\paapi5\v1\ProductAdvertisingAPIClientException;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelTrait;
use GuzzleHttp\ClientInterface;

/**
 * Service class Amazon
 */
class Amazon {

  use LoggerChannelTrait;

  /**
   * Mutable config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $config;

  /**
   * The HTTP client to fetch the files with.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The LoggerInterface provided by Drupal core.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Create instance of Amazon class.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   * @param \GuzzleHttp\ClientInterface $http_client
   */
  public function __construct(ConfigFactoryInterface $configFactory, ClientInterface $http_client) {
    $this->config = $configFactory->getEditable('amazon.settings');
    $this->httpClient = $http_client;
    $this->logger = $this->getLogger('advertising_products_amazon');
  }

  /**
   * Extract ASIN from given url.
   *
   * @param string $url
   * @return mixed
   */
  public function getAsinFromUri($url) {

    // regex from http://littlegreenfootballs.com/article/44256_Tech_Note-_A_Regular_Expression_to_Extract_the_ASIN_Product_Code_From_Any_Amazon_URL
    $regex = '~
    (?:www\.)?              # optionally starts with www.
    ama?zo?n\.              # also allow shortened amzn.com URLs
    (?:
        com                 # match all Amazon domains
        |
        ca
        |
        co\.uk
        |
        co\.jp
        |
        de
        |
        fr
    )
    /
    (?:                     # here comes the stuff before the ASIN
        exec/obidos/ASIN/   # the possible components of a URL
        |
        o/
        |
        gp/product/
        |
        (?:                 # the dp/ format may contain a title
            (?:[^"\'/]*)/   # anything but a slash or quote
        )?                  # optional
        dp/
        |                   # if short format, nothing before ASIN
    )
    ([A-Z0-9]{10})          # capture group $1 contains the ASIN
    (?:                     # everything after the ASIN
        (?:/|\?|\#)         # beginning with /, ? or #
        (?:[^"\'\s]*)       # everything up to quote or white space
    )?                      # optional
~isx';

    if (preg_match($regex, $url, $matches)) {
      return $matches[1];
    }
    return FALSE;
  }

  /**
   * Fetch product data from API.
   *
   * @param string $product_id
   * @return array|boolean
   *
   * @throws \Exception
   */
  public function itemLookup($product_id) {
    $accessKey = $this->config->get('access_key');
    $accessSecret = $this->config->get('access_secret');
    $associateTag = $this->config->get('associates_id');

    // Amazon API configuration is required.
    if (!isset($accessKey) || !isset($accessSecret) || !isset($associateTag)) {
      $this->logger->critical('Missing Amazon API credentials.');
      return FALSE;
    }

    // Prepare Amazon API configuration object.
    $config = new Configuration();
    $config->setAccessKey($accessKey);
    $config->setSecretKey($accessSecret);
    $config->setHost('webservices.amazon.de');
    $config->setRegion('eu-west-1');

    // Prepare Amazon API request object.
    $apiInstance = new DefaultApi($this->httpClient, $config);
    $resources = [
      GetItemsResource::BROWSE_NODE_INFOBROWSE_NODESANCESTOR,
      GetItemsResource::IMAGESPRIMARYLARGE,
      GetItemsResource::IMAGESVARIANTSLARGE,
      GetItemsResource::ITEM_INFOTITLE,
      GetItemsResource::ITEM_INFOBY_LINE_INFO,
      GetItemsResource::ITEM_INFOTRADE_IN_INFO,
      GetItemsResource::OFFERSLISTINGSAVAILABILITYMESSAGE,
      GetItemsResource::OFFERSLISTINGSPRICE,
      GetItemsResource::OFFERSLISTINGSSAVING_BASIS,
    ];

    # Forming the Amazon API request.
    $getItemsRequest = new GetItemsRequest();
    $getItemsRequest->setItemIds([$product_id]);
    $getItemsRequest->setPartnerTag($associateTag);
    $getItemsRequest->setPartnerType(PartnerType::ASSOCIATES);
    $getItemsRequest->setResources($resources);

    # Validating getItemsRequest via Amazon API.
    $invalidPropertyList = $getItemsRequest->listInvalidProperties();
    $length = count($invalidPropertyList);
    if ($length > 0) {
      $this->logger->critical('Error forming the Amazon API request');
      return FALSE;
    }

    // Do the Amazon API request.
    try {
      $getItemsResponse = $apiInstance->getItems($getItemsRequest);
      if ($getItemsResponse->getItemsResult() != null) {
        return $this->parseResponse($getItemsResponse->getItemsResult()->getItems());
      }
    }
    catch (ApiException $e) {
      $this->logger->error('HTTP status code: ' . $e->getCode() . ' - Error message: ' . $e->getMessage());
      if ($e->getResponseObject() instanceof ProductAdvertisingAPIClientException) {
        $errors = $e->getResponseObject()->getErrors();
        foreach ($errors as $error) {
          $this->logger->error('Error code: ' . $error->getCode() . ' - Error message: ' . $error->getMessage());
        }
      }
    }
    catch (\Exception $e) {
      $this->logger->error($e->getMessage());
    }
    return FALSE;
  }

  /**
   * Fetch primary image from API.
   *
   * @param \Amazon\ProductAdvertisingAPI\v1\com\amazon\paapi5\v1\Item $product
   *   A product item fetched from the Amazon API.
   *
   * @return \Psr\Http\Message\ResponseInterface|bool
   *   Response from server or FALSE.
   */
  public function retrievePrimaryImage($product) {
    try {
      $image_path = (string) $product->getImages()->getPrimary()->getLarge()->getURL();
      return $this->retrieveImage($image_path, $product->getASIN());
    }
    catch (\Exception $e) {
      $this->logger->error($e->getMessage());
    }
    return FALSE;
  }

  /**
   * Fetch product image from Amazon API.
   *
   * @param $image_path
   *   A URL to the image.
   * @param $ASIN
   *  The ASIN of the product.
   *
   * @return mixed|\Psr\Http\Message\ResponseInterface|bool
   *   Response from server or FALSE.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function retrieveImage($image_path, $ASIN) {
    if ($image_path) {
      $tries = 0;
      do {
        $tries++;
        $image = \Drupal::httpClient()->request('GET', $image_path);
      } while (($image->getStatusCode() != 200 || !$image->getBody()) && $tries < 3);

      if (!$image->getBody()) {
        $error_msg = 'Error Message: ' . $image->getStatusCode() ? $image->getStatusCode() : "Couldn't retrieve image";
        throw new \Exception($error_msg, $ASIN, 'original');
      }

      if (!in_array($image->getHeader('content-type')[0], array('image/png', 'image/jpeg'))) {
       $error_msg = 'Error Message: Unexpected content type "' . $image->getHeader('content-type') . '"';
       throw new \Exception($error_msg, $ASIN, 'original');
      }
    }
    else {
       $this->logger->error('No image path found for ASIN "%s"', ['%s' => $ASIN]);
    }

    if ($image) {
      return $image;
    }
    $this->logger->error('Could not find image for ASIN "%s"', ['%s' => $ASIN]);
    return FALSE;
  }

  /**
   * Returns the array of items mapped to ASIN
   *
   * @param array $items Items value.
   *
   * @return array of \Amazon\ProductAdvertisingAPI\v1\com\amazon\paapi5\v1\Item mapped to ASIN.
   */
  function parseResponse($items) {
    $mappedResponse = array();
    foreach ($items as $item) {
      $mappedResponse[$item->getASIN()] = $item;
    }
    return $mappedResponse;
  }

  /**
   * Extracts extra image URLs from Amazon API product.
   *
   * @param \Amazon\ProductAdvertisingAPI\v1\com\amazon\paapi5\v1\Item $product
   *   A product item fetched from the Amazon API.
   *
   * @return array
   *   URLs of extra images
   */
  public function prepareExtraImageInfo($product) {
    $all_images = [];
    if ($product->getImages()->getVariants() != NULL) {
      foreach ($product->getImages()->getVariants() as $variantImage) {
        if (($variantImage->getLarge()->getURL() != NULL)
          && parse_url($variantImage->getLarge()->getURL())) {
          $extra_image_url = (string) $variantImage->getLarge()->getURL();
          $all_images[$extra_image_url] = $extra_image_url;
        }
      }
    }
    return $all_images;
  }

  /**
   * Override amazon.settings config.
   *
   * @param array $config
   *
   * @return \Drupal\Core\Config\ConfigFactoryInterface
   */
  public function setConfig(array $config) {
    foreach ($config as $key => $value) {
      if (!empty($value)) {
        $this->config->set($key, $value);
      }
    }
    return $this->config;
  }

  /**
   * Store info about all the images.
   *
   * @param int $entity_id
   *   A product entity id.
   * @param array $urls
   *   An array of image urls.
   *
   * @throws \Exception
   *   If storing failed.
   */
  public function storeExtraImageInfo($entity_id, $urls) {
    // Check for data we have already stored.
    if (count($urls)) {
      $query = \Drupal::database()->select('advertising_products_amazon_image_data', 'i');
      $query->fields('i', ['iid', 'url']);
      $query->condition('pid', $entity_id);
      $query->condition('url', $urls, 'IN');
      $already_there = $query->execute()->fetchAllAssoc('iid');
      // Filter out the pre-existing values.
      $new_urls = $urls;
      foreach ($already_there as $idx => $values) {
        if (isset($new_urls[$values->url])) {
          unset($new_urls[$values->url]);
        }
      }
      // Insert any new urls.
      foreach ($new_urls as $url) {
        $insert = \Drupal::database()->insert('advertising_products_amazon_image_data');
        $insert->fields(['pid', 'url']);
        $insert->values([$entity_id, $url]);
        $insert->execute();
      }
    }
  }
}
