<?php

namespace Drupal\advertising_products_amazon\Plugin\AdvertisingProducts\Provider;

use Drupal\advertising_products\AdvertisingProductsProviderBase;
use Drupal\advertising_products_amazon\Amazon;
use Drupal\Component\Utility\Unicode;
use Drupal\Core\Entity\EntityManagerInterface;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\taxonomy\Entity\Term;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides advertising products provider plugin for Amazon.
 *
 * @AdvertisingProductsProvider(
 *   id = "amazon_provider",
 *   name = @Translation("Amazon product provider")
 * )
 */
class AmazonProvider extends AdvertisingProductsProviderBase {

  use LoggerChannelTrait;

  /**
   * @var array
   */
  public static $providerDomains = ['www.amazon.de', 'www.amazon.com'];

  /**
   * @var string
   */
  public static $productBundle = 'advertising_product_amazon';

  /**
   * @var \Drupal\advertising_products_amazon\Amazon
   */
  protected $amazonService;

  /**
   * The LoggerInterface provided by Drupal core.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /** @var  string */
  protected $vocabulary;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityManagerInterface $entityManager, Amazon $amazonService) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entityManager);
    $this->amazonService = $amazonService;
    $this->vocabulary = \Drupal::config('advertising_products_amazon.settings')->get('vocabulary');
    $this->logger = $this->getLogger('advertising_products_amazon');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity.manager'),
      $container->get('advertising_products_amazon.amazon')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getProductIdFromUrl($url) {
    if ($product_id = $this->amazonService->getAsinFromUri($url)) {
      return $product_id;
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function fetchProductOnTheFly($product_id, $entity_id = NULL, $consumer = NULL) {

    // Load advertising_product entity.
    // TODO: Why we do not reference AdvertisingProductEntity instead of entity_id?
    if (!is_null($entity_id)) {
      $entityType = 'advertising_product';
      $query = \Drupal::entityQuery($entityType)
        ->condition('product_provider', $this->getPluginId())
        ->condition('id', $entity_id)
        ->condition('product_id', $product_id);

      $ids = $query->execute();
      if (count($ids)) {
        $entities = \Drupal::entityTypeManager()->getStorage($entityType)->loadMultiple($ids);
        $advertisingProduct =  reset($entities);
        return $advertisingProduct;
      }
    }

    // Override Amazon API credentials given by consumer entity.
    // TODO: Move this stuff to a method like prepareAmazonService() or so.
    $moduleHandler = \Drupal::service('module_handler');
    if ($moduleHandler->moduleExists('consumers') && $consumer) {
      $amazonConfig = [
        'associates_id' => $consumer->amazon_associates_tag->value,
        'access_key' => $consumer->amazon_access_key->value,
        'access_secret' => $consumer->amazon_access_secret->value,
        'locale' => $consumer->amazon_locale->value,
      ];
      $this->amazonService->setConfig($amazonConfig);
    }

    // Retrieve Amazon product.
    try {
      if ($product_request = $this->queryProduct($product_id)) {
        if (isset($product_request[$product_id])) {
          if(!isset($entity_id)) {
            $entity_id = $this->getEntityIdFromProductId($product_id);
          }
          return $this->saveProduct($product_request[$product_id], $entity_id);
        }
      }
    }
    catch (\Exception $e) {
      $this->logger->critical($e->getMessage());
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function queryProduct($product_id) {
    try {
      // Retrieve product information from Amazon API.
      return $this->amazonService->itemLookup($product_id);
    }
    catch (\Exception $e) {
      $this->logger->critical($e->getMessage());
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function searchProduct($input) {
  }

  public function getImagePrefix($product_data) {
    $parts = [
      'amazon',
      $product_data->ASIN,
      // We need a random string, to prevent browser caching for changing images.
      uniqid()
    ];
    return implode('-', $parts);
  }

  /**
   * {@inheritdoc}
   */
  public function saveProduct($product_data, $entity_id = NULL) {

    /** @var \Amazon\ProductAdvertisingAPI\v1\com\amazon\paapi5\v1\Item $product_data */
    if ($entity_id) {
      // Update an existing product entity.
      $product = $this->entityManager->getStorage('advertising_product')->load($entity_id);
    }
    else {
      // Create a new product entity.
      $item['type'] = $this::$productBundle;
      $item['product_provider'] = $this->getPluginId();
      $item['product_id'] = (string) $product_data->getASIN();
      $product = $this->entityManager->getStorage('advertising_product')->create($item);
    }

    /** @var \Drupal\advertising_products\Entity\AdvertisingProduct $product */
    $product->product_name->value = Unicode::substr((string) $product_data->getItemInfo()->getTitle()->getDisplayValue(), 0, 255);
    $product->product_description->value = '';
    $brand = $product_data->getItemInfo()->getByLineInfo()->getBrand();
    if ($brand) {
      $product->product_brand->value = Unicode::substr((string) $brand->getDisplayValue(), 0, 50);
    }
    else {
      $product->product_brand->value = '';
    }
    $product->product_url->uri = (string) $product_data->getDetailPageURL();
    $product->product_url->options = [];
    $product->product_shop->value = 'Amazon';
    $product->status->value = 1;  // TODO: Try to add a 'in stock' update.

    // Set price and original_price from RepsonseGroup 'Offers.Listings.SavingBasis'.
    if ($product_data->getOffers()) {
      if ($product_data->getOffers()->getListings()[0]->getSavingBasis() != NULL) {
        $list_price = (float) $product_data->getOffers()->getListings()[0]->getSavingBasis()->getAmount();
        $product->product_price->value = $list_price;
        $product->product_original_price->value = $list_price;
        $product->product_currency->value = (string) $product_data->getOffers()->getListings()[0]->getSavingBasis()->getCurrency();
      }

      // Set price and original_price from ResourceGroup 'Offers.Listings'.
      $product->product_price->value = (float) $product_data->getOffers()->getListings()[0]->getPrice()->getAmount();
      $product->product_currency->value = (string) $product_data->getOffers()->getListings()[0]->getPrice()->getCurrency();
      if ($product->product_original_price->isEmpty()) {
        $product->product_original_price->value = $product->product_price->value;
      }
      // check for availability
      $availability = (string) $product_data->getOffers()->getListings()[0]->getAvailability()->getMessage();
      switch ($availability) {
        case '':
        case 'Not yet released':
        case 'Not yet published ':
        case 'This item is not stocked or has been discontinued.':
        case 'Out of Stock':
        case 'Limited Availability':
        case 'Out of Print--Limited Availability':
        case 'Special Order':
        case 'This item is currently not available by this merchant':
          $product->product_sold_out->value = TRUE;
          break;
        default:
          $product->product_sold_out->value = FALSE;
          break;
      }
    }
    else {
      $product->product_sold_out->value = TRUE;
    }

    // Add the product_data to allow other modules to use it in hooks.
    $product->product_data = $product_data;

    // Add categories from resource 'BrowseNodeInfo.BrowseNodes.Ancestor'.
    if ($this->vocabulary && $product->hasField('field_category')) {
      $categories = $this->retrieveCategories($product_data);
      if (count($categories)) {
        $product->set('field_category', $categories);
      }
    }

    // We need to update the changed timestamp and save to get ID.
    $product->changed->value = \Drupal::time()->getRequestTime();
    $product->save();

    // Get and store extra images.
    $extraImageInfo = $this->amazonService->prepareExtraImageInfo($product_data);
    $this->amazonService->storeExtraImageInfo($product->id(), $extraImageInfo);

    // Retrieve primary product image, only if is an new product.
    if (!$entity_id) {
      try {
        if ($image = $this->amazonService->retrievePrimaryImage($product_data)) {
          if ($file = $this->saveImage($image, $product_data)) {
            $product->product_image->target_id = $file->id();
            $product->product_image->alt = Unicode::substr((string) $product_data->getItemInfo()->getTitle()->getDisplayValue(), 0, 512);
            $product->save();
            return $product;
          }
        }
      }
      catch (\Exception $e) {
        $this->logger->critical($e->getMessage());

        // We do not want to have products without images.
        $product->delete();
        return FALSE;
      }
    }
    return $product;
  }

  /**
   * {@inheritdoc}
   */
  public function updateProduct($product_id, $entity_id, $consumer = NULL) {
    // Override Amazon API credentials given by consumer entity.
    // TODO: Move this stuff to a method like prepareAmazonService() or so.
    $moduleHandler = \Drupal::service('module_handler');
    if ($moduleHandler->moduleExists('consumers') && $consumer) {
      $amazonConfig = [
        'associates_id' => $consumer->amazon_associates_tag->value,
        'access_key' => $consumer->amazon_access_key->value,
        'access_secret' => $consumer->amazon_access_secret->value,
        'locale' => $consumer->amazon_locale->value,
      ];
      $this->amazonService->setConfig($amazonConfig);
    }

    // Retrieve product data from Amazon API.
    if ($product_request = $this->queryProduct($product_id)) {
      if (isset($product_request[$product_id])) {
        $this->saveProduct($product_request[$product_id], $entity_id);
        return TRUE;
      }
      else {
        $this->setProductInactive($entity_id);
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function setProductInactive($entity_id) {
    if ($product = $this->entityManager->getStorage('advertising_product')->load($entity_id)) {
      $product->product_sold_out->value = TRUE;
      $product->save();
    }
  }


  /**
   * Extract categories from amazon response and map them to Drupal terms.
   *
   * @param \Amazon\ProductAdvertisingAPI\v1\com\amazon\paapi5\v1\Item $product
   *   A product array fetched from the api
   * @return array $categories
   *   Response from api
   * @throws \Exception
   */
  public function retrieveCategories($product) {
    $all_categories = [];
    if ($product->getBrowseNodeInfo()->getBrowseNodes() != NULL) {
      foreach ($product->getBrowseNodeInfo()->getBrowseNodes() as $browseNode) {
        $categories = $this->getCategories($browseNode);
        if ($categories) {
          $all_categories[] =$categories;
        }
      }
    }

    $term_ids = [];
    foreach ($all_categories as $category) {
      $id = $this->getTerms($category);
      if ($id) {
        $term_ids[] = $id;
      }
    }
    return $term_ids;
  }

  /**
   * @param \Amazon\ProductAdvertisingAPI\v1\com\amazon\paapi5\v1\BrowseNode|\Amazon\ProductAdvertisingAPI\v1\com\amazon\paapi5\v1\BrowseNodeAncestor $browseNode
   *
   * @return array|bool
   */
  public function getCategories($browseNode) {
    // Amazon's way of handling categories is a bit strange.
    // In addition to the normal category tree, it has several others, where
    // the uppermost parent has an empty name.
    // To avoid adding those categories, we return FALSE here.
    if ($browseNode->getDisplayName() === NULL) {
      return FALSE;
    }
    $categories = [
      'id' => (string) $browseNode->getId(),
      'name' => (string) $browseNode->getContextFreeName(),
      'parent' => '',
    ];
    if ($browseNode->getAncestor() != NULL) {
      $parent = $this->getCategories($browseNode->getAncestor());
      if ($parent) {
        $categories['parent'] = $parent;
      }
      else {
        return FALSE;
      }
    }
    return $categories;
  }

  /**
   * This function retrieves the lowest term of the BrowseNode tree
   *
   * It creates it and its ancestors if it is not found.
   */
  public function getTerms($category) {
    $term_id = 0;
    $term = advertising_products_find_term($this->vocabulary, $category['id']);
    if (!$term && isset($category['name']) && strlen($category['name']) > 0) {
      $term = Term::create(
        [
          'vid' => $this->vocabulary,
          'name' => $category['name'],
          'field_original_id' => $category['id'],
          'status' => 0,
        ]
      );
      $term->save();
      $term_id = $term->id();
    }
    else if ($term) {
      $term_id = $term->id();
    }
    if (is_array($category['parent'])) {
      $parent_term_id = $this->getTerms($category['parent']);
      $term->set('parent', $parent_term_id);
      $term->save();
    }
    return $term_id;
  }

  /**
   * {@inheritdoc}
   *
   * Save image selection
   */
  public function submitFieldWidget(array $values) {
    if ($values['image_selection_input']) {
      $product = \Drupal::entityTypeManager()->getStorage('advertising_product')->load($values['target_id']);

      if ($image = $this->amazonService->retrieveImage($values['image_selection_input'], $product->product_id->value)) {
        $poduct_data = new \stdClass();
        $poduct_data->ASIN = $product->product_id->value;
        if ($file = $this->saveImage($image, $poduct_data)) {
          $product->product_image->target_id = $file->id();
          $product->save();
        }
      }
    }
  }

}
